package com.lovre.refs.biblo.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.HeadlessException;
import java.awt.Window;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.LinkedList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;

import com.lovre.refs.biblo.utils.Formatter;
import com.lovre.refs.biblo.utils.Globals;

public class GUI extends JFrame {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	protected JButton bibloButton;
	
	protected JButton automaticButton;
	
	protected DropButton pathButton;
	
	protected JTextField inputTextField;
	
	protected JLabel inputChangeLabel;
	
	protected JTextField outputTextField;
	
	protected JTextField mapTextField;
	
	protected JLabel mapChangeLabel;
	
	protected JTextField discaredTextField;
	
	protected JLabel summaryLabel;

	public GUI() throws HeadlessException {
		super();
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		init();
		
		Globals.CHANGING = new Thread(new Runnable() {
			@Override
			public void run() {
				while (true) {
					if (Globals.CHANGING == null)
						break;
					
					String path = pathButton.getText().equals("Select source path...")? ".": pathButton.getText();
					
					File input = new File(path + "/" + inputTextField.getText());
					if (inputChangeLabel.getText().length() == 0 && input.lastModified() > Globals.TIC)
						inputChangeLabel.setText(" Changed at " + new SimpleDateFormat("h:mm:ss a").format(Calendar.getInstance().getTime()));
					
					File map = new File(path + "/" + mapTextField.getText());
					if (mapChangeLabel.getText().length() == 0 && map.lastModified() > Globals.TIC)
						mapChangeLabel.setText(" Changed at " + new SimpleDateFormat("h:mm:ss a").format(Calendar.getInstance().getTime()));
					
					try {
						Thread.sleep(Globals.SLEEP_MILLIS);
					} catch (InterruptedException e) {
						summaryLabel.setText("<html><font color=#" + Globals.THEME_COLOR.getRGB() + "><b>" + e.getMessage() + "</b></font></html>");
						break;
					}
					
					repaint();
				}
			}
		});
		
		Globals.CHANGING.start();
	}

	private void init() {
		setTitle("BibLo");
		setLocation(128, 0);
		setPreferredSize(new Dimension(480, 352));
		getContentPane().setLayout(new BorderLayout());
		
		getRootPane().putClientProperty("apple.awt.brushMetalLook", true);
		
		/*
	     * Navigation panel.
	     */
	   
		JPanel navigationPanel = new JPanel();
		getContentPane().add(navigationPanel, BorderLayout.NORTH);

		bibloButton = manage(new JButton(new ImageIcon("icons/BibLo.png")));
		bibloButton.setPreferredSize(new Dimension(40, 40));
		bibloButton.setToolTipText("Execute BibLo");
		bibloButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				try {
					List<String> discarded = new LinkedList<String>();
					for (String discard: discaredTextField.getText().split(","))
						discarded.add(discard.trim());
					
					String path = pathButton.getText().equals("Select source path...")? ".": pathButton.getText();
					summaryLabel.setText(Formatter.format(path + "/" + inputTextField.getText(), path + "/" + outputTextField.getText(), path + "/" + mapTextField.getText(), discarded));
					
					executed();
				} catch (Exception e) {
					summaryLabel.setText("<html><font color=#" + Globals.THEME_COLOR.getRGB() + "><b>" + e.getMessage() + "</b></font></html>");
				}
			}
		});
		navigationPanel.add(bibloButton);

		automaticButton = manage(new JButton("Automatic"));
		automaticButton.setPreferredSize(new Dimension(84, 32));
		automaticButton.setToolTipText("Automatic BibLo");
	    automaticButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				boolean automatic = automaticButton.getText().equals("Automatic");
				
				automatic(automatic);
				
				if (automatic) {
					Globals.AUTOMATIC = new Thread(new Runnable() {
						@Override
						public void run() {
							long tic = -1;
							while (true) {
								if (Globals.AUTOMATIC == null)
									break;
								
								String path = pathButton.getText().equals("Select source path...")? ".": pathButton.getText();
								File input = new File(path + "/" + inputTextField.getText());
								File map = new File(path + "/" + mapTextField.getText());
								if (Math.max(input.lastModified(), map.lastModified()) > tic) {
									tic = Math.max(input.lastModified(), map.lastModified());
									
									while (true) {
										try {
											Thread.sleep(Globals.DELAY_MILLIS);
										} catch (InterruptedException e) {
											summaryLabel.setText("<html><font color=#" + Globals.THEME_COLOR.getRGB() + "><b>" + e.getMessage() + "</b></font></html>");
											break;
										}
										
										if (tic == Math.max(input.lastModified(), map.lastModified()))
											break;
										tic = Math.max(input.lastModified(), map.lastModified());
									}
									
									List<String> discarded = new LinkedList<String>();
									for (String discard: discaredTextField.getText().replace(" ", "").split(","))
										discarded.add(discard.trim());
									
									try {
										summaryLabel.setText(Formatter.format(path + "/" + inputTextField.getText(), path + "/" + outputTextField.getText(), path + "/" + mapTextField.getText(), discarded));
										
										executed();
									} catch (Exception e) {
										summaryLabel.setText("<html><font color=#" + Globals.THEME_COLOR.getRGB() + "><b>" + e.getMessage() + "</b></font></html>");
									}
									
									tic = Math.max(input.lastModified(), map.lastModified());
								}
								
								try {
									Thread.sleep(Globals.SLEEP_MILLIS);
								} catch (InterruptedException e) {
									summaryLabel.setText("<html><font color=#" + Globals.THEME_COLOR.getRGB() + "><b>" + e.getMessage() + "</b></font></html>");
									break;
								}
							}
						}
					});
					
					Globals.AUTOMATIC.start();
				}
				else
					Globals.AUTOMATIC = null;
				
				if (automatic) {
					automaticButton.setText("Standard");
					automaticButton.setToolTipText("Standard BibLo");
				}
				else {
					automaticButton.setText("Automatic");
					automaticButton.setToolTipText("Automatic BibLo");
				}
				
				automaticButton.setEnabled(true);
			}
		});
	    navigationPanel.add(automaticButton);
		
	    /*
	     * Main panel.
	     */
	    
		JPanel mainPanel = new Frame();
		mainPanel.setLayout(new BorderLayout());
		getContentPane().add(mainPanel, BorderLayout.CENTER);
	    
	    /*
	     * BibLo panel.
	     */
		
	    JPanel bibloPanel = new JPanel();
	    bibloPanel.setBorder(BorderFactory.createTitledBorder(new LineBorder(Color.BLACK, 1), "", TitledBorder.LEFT, TitledBorder.DEFAULT_POSITION, Globals.TITLE_FONT, Globals.THEME_COLOR));
	    bibloPanel.setBackground(Globals.BACKGROUND_COLOR);
	    mainPanel.add(bibloPanel, BorderLayout.NORTH);
	    
	    GridLayout bibloLayout = new GridLayout(5, 3);
	    bibloLayout.setVgap(6);
	    bibloPanel.setLayout(bibloLayout);
	    
	    JLabel pathLabel = manage(new JLabel("Source path: ", SwingConstants.RIGHT));
	    bibloPanel.add(pathLabel);
	    
	    pathButton = (DropButton)manage(new DropButton("Select source path..."));
		pathButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				try {
					JFileChooser chooser = new JFileChooser(new File(pathButton.getText().equals("Select source path...")? "~": pathButton.getText()).getParent());
					chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
					if (chooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
						pathButton.setText(chooser.getSelectedFile().getAbsolutePath().replace("\\", "/"));
						pathButton.setToolTipText(pathButton.getText());
					}
				} catch (Exception e) {
					summaryLabel.setText("<html><font color=#" + Globals.THEME_COLOR.getRGB() + "><b>" + e.getMessage() + "</b></font></html>");
				}
			}
		});
		bibloPanel.add(pathButton);
		
		bibloPanel.add(new JLabel(""));
		
		JLabel inputLabel = manage(new JLabel("BibTeX input: ", SwingConstants.RIGHT));
	    bibloPanel.add(inputLabel);
		
		inputTextField = manage(new JTextField("biblo.bib", SwingConstants.CENTER));
		bibloPanel.add(inputTextField);
		
		inputChangeLabel = manage(new JLabel("", SwingConstants.LEFT));
		inputChangeLabel.setFont(Globals.COMMENT_FONT);
	    bibloPanel.add(inputChangeLabel);
		
		JLabel outputLabel = manage(new JLabel("BibTeX output: ", SwingConstants.RIGHT));
	    bibloPanel.add(outputLabel);

	    outputTextField = manage(new JTextField("bibliography.bib", SwingConstants.CENTER));
		bibloPanel.add(outputTextField);
		
		bibloPanel.add(new JLabel(""));
		
		JLabel mapLabel = manage(new JLabel("BibLo map: ", SwingConstants.RIGHT));
	    bibloPanel.add(mapLabel);
	    	
	    mapTextField = manage(new JTextField("biblo.map", SwingConstants.CENTER));
		bibloPanel.add(mapTextField);
		
		mapChangeLabel = manage(new JLabel("", SwingConstants.LEFT));
		mapChangeLabel.setFont(Globals.COMMENT_FONT);
	    bibloPanel.add(mapChangeLabel);
		
		JLabel discaredLabel = manage(new JLabel("BibTeX discard: ", SwingConstants.RIGHT));
	    bibloPanel.add(discaredLabel);
	    
	    discaredTextField = manage(new JTextField("note, annote, keywords, url, file, doi, issn", SwingConstants.CENTER));
		bibloPanel.add(discaredTextField);
		
		bibloPanel.add(new JLabel(""));
	    
	    /*
	     * Summary panel.
	     */
		
	    JPanel summaryPanel = new JPanel();
	    summaryPanel.setLayout(new BorderLayout());
	    mainPanel.add(summaryPanel, BorderLayout.CENTER);
	    
	    summaryLabel = manage(new JLabel(" "));
	    summaryLabel.setHorizontalAlignment(SwingConstants.CENTER);
	    summaryPanel.add(summaryLabel, BorderLayout.CENTER);
	    
		pack(); 
	}
	
	protected static JButton manage(JButton button) {
		button.setFocusable(false);
		button.setFont(Globals.BASIC_FONT);
		
		return button;
	}
	
	protected static JTextField manage(JTextField textField) {
		textField.setFont(Globals.BASIC_FONT);
		
		return textField;
	}
	
	protected static JLabel manage(JLabel label) {
		label.setFont(Globals.BASIC_FONT);
		
		return label;
	}

	protected void automatic(boolean automatic) {
		bibloButton.setEnabled(!automatic);
		automaticButton.setEnabled(!automatic);
		pathButton.setEnabled(!automatic);
		inputTextField.setEnabled(!automatic);
		outputTextField.setEnabled(!automatic);
		mapTextField.setEnabled(!automatic);
		discaredTextField.setEnabled(!automatic);
	}
	
	protected void executed() {
		inputChangeLabel.setText("");
		mapChangeLabel.setText("");
		
		Globals.TIC = System.currentTimeMillis();
	}
	
}

class Frame extends JPanel {

    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	public Frame() {
    	super();
    	
        setOpaque(false);

        setBorder(BorderFactory.createEmptyBorder());
    }

    @Override
    public Border getBorder() {
        Window window = SwingUtilities.getWindowAncestor(this);
        return window != null && window.isFocused()? BorderFactory.createMatteBorder(0, 0, 1, 0, Globals.FOCUSED_TOOLBAR_COLOR): BorderFactory.createMatteBorder(0, 0, 1, 0, Globals.UNFOCUSED_TOOLBAR_COLOR);
    }

}

class DropButton extends JButton implements DropTargetListener {
    
    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;

    public DropButton(String text) {
    	super(text);
    	
    	new DropTarget(this, this);
    }

    public void dragEnter(DropTargetDragEvent event) {

    }

    public void dragOver(DropTargetDragEvent event) {

    }

    public void dragExit(DropTargetEvent event) {

    }

    @SuppressWarnings("unchecked")
	public void drop(DropTargetDropEvent event) {
    	File folder = null;
    	
        Transferable transferable = event.getTransferable();
        if (transferable.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
        	event.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
        	
            try {
            	folder = ((List<File>)transferable.getTransferData(DataFlavor.javaFileListFlavor)).iterator().next();
            }
            catch(Exception e) {
                event.rejectDrop();
                
                return;
            }

            event.getDropTargetContext().dropComplete(true);
        }
        else
            event.rejectDrop();
        
        if (folder != null && folder.exists()) {
        	setText(folder.getAbsolutePath().replace("\\", "/"));
            setToolTipText(getText());
            
            update();
        }
    }

    public void dropActionChanged(DropTargetDragEvent event) {

    }
    
    public void update() {
    	
    };
    
}