package com.lovre.refs.biblo.utils;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class Formatter {

	public static String format(String input, String output, String map, List<String> discarded) throws Exception {
		return format(new File(input), new File(output), new File(map), discarded);
	}
	
	public static String format(File input, File output, File map, List<String> discarded) throws Exception {
		Map<String, String> mapping = read(map);
		
		int count = 0;
		Map<String, List<List<String>>> references = new HashMap<String, List<List<String>>>();
		BufferedReader reader = new BufferedReader(new FileReader(input));
		
		List<String> ref;
		while ((ref = read(reader)).size() > 0) {
			String key = key(ref);
			if (references.get(key) == null)
				references.put(key, new LinkedList<List<String>>());
			references.get(key).add(ref);
			count++;
		}
		reader.close();
		
		BufferedWriter writer = new BufferedWriter(new FileWriter(output));
		for (String key: references.keySet())
			if (references.get(key).size() > 1) {
				Set<String> used = new HashSet<String>();
				for (List<String> reference: references.get(key)) {
					String suffix = mapping.get(key + " # " + title(reference));
					if (suffix != null)
						used.add(suffix);
				}
				char free = 'a';
				for (List<String> reference: references.get(key)) {
					String identifier = key + " # " + title(reference);
					String suffix = mapping.get(identifier);
					if (suffix == null) {
						while (used.contains("" + free))
							free++;
						suffix = "" + free;
						mapping.put(identifier, suffix);
						used.add(suffix);
					}
					for (Iterator<String> iterator = reference.iterator(); iterator.hasNext(); ) {
						String r = iterator.next();
						for (String entry: discarded) 
							if (r.startsWith(entry + " = {")) { 
								iterator.remove();
								break;
							}	
					}
					write(reference, key + suffix, writer);	
				}
			}
			else {
				for (Iterator<String> iterator = references.get(key).get(0).iterator(); iterator.hasNext(); ) {
					String r = iterator.next();
					for (String entry: discarded)
						if (r.startsWith(entry + " = {")) { 
							iterator.remove();
							break;
						}
				}
				write(references.get(key).get(0), key, writer);
			}
		writer.flush();
		writer.close(); 
		
		write(map, references, mapping);
		
		String report = "<html><center>" +
					"<font color=#" + Globals.THEME_COLOR.getRGB() + "><b>References: " + count + "</b></font><br>" +
					"Unique keys: " + (count - mapping.size()) + "<br>" +
					"Map keys: " + (references.size() - count + mapping.size()) + " (" + mapping.size() + ")<br>" +
					"<br>" +
					"<font color=#" + Globals.THEME_COLOR.getRGB() + "><b>" + new SimpleDateFormat("E, dd MMM, yyyy, h:mm:ss a").format(Calendar.getInstance().getTime()) + "</b></font>" +
					"</center></html>"; 
		return report;
	}
	
	private static List<String> read(BufferedReader reader) throws Exception {
		String line;
		List<String> reference = new LinkedList<String>();
		while ((line = reader.readLine()) != null) {
			line = line.trim();
			if (line.length() > 0)
				if (line.startsWith("}")) {
					reference.add("}");
					break;
				}
				else if (line.startsWith("@") || reference.size() > 0)
					reference.add(line.endsWith(",")? line.substring(0, line.length() - 1): line);
		}
		return reference;
	}
	
	private static void write(List<String> reference, String key, BufferedWriter writer) throws Exception {
		reference.set(0, reference.get(0).split("\\{")[0] + "{" + key);
		for (int i = 0; i < reference.size(); i++) {
			writer.write(reference.get(i) + (i < reference.size() - 2? ",": ""));
			writer.newLine();
		}
		writer.newLine();
	}
	
	private static String key(List<String> reference) {
		String key = "";
		
		String authors = authors(reference);
		if (authors.contains(" and ")) {
			String[] array = authors.split(" and ");
			for (int i = 0; i < array.length; i++) {
				int index = 0;
				while (!Character.isLetter(array[i].charAt(index)) || !Character.isUpperCase(array[i].charAt(index)))
					index++;
				if (index < array[i].length())
					key += array[i].charAt(index);
			}
			key = key.toUpperCase();
		}
		else {
			int index = 0;
			while (key.length() < 3) {
				while (!Character.isLetter(authors.charAt(index)) || key.length() == 0 && Character.isLowerCase(authors.charAt(index)) )
					index++;
				key += authors.charAt(index++);
				if (index >= authors.length() || Character.isWhitespace(authors.charAt(index)))
					break;
			}
		}
		
		key += year(reference).substring(2);

		return key;
	}
	
	private static String authors(List<String> reference) {
		String authors = "Authors";
		int i = 0;
		while (i < reference.size()) {
			if (reference.get(i).startsWith("author = {")) {
				authors = reference.get(i).substring(10, reference.get(i).length() - 1).trim();
				break;
			}
			i++;
		}
		return authors;
	}
	
	private static String title(List<String> reference){
		String title = "Title";
		int i = 0;
		while (i < reference.size()) {
			if (reference.get(i).startsWith("title = {")) {
				title = reference.get(i).substring(9, reference.get(i).length() - 1).trim();
				break;
			}
			i++;
		}
		return title;
	}
	
	private static String year(List<String> reference){
		String year = "1900";
		int i = 0;
		while (i < reference.size()) {
			if (reference.get(i).startsWith("year = {")) {
				year = reference.get(i).substring(8).trim();
				while (!Character.isDigit(year.charAt(year.length() - 1)))
					year = year.substring(0, year.length() - 1);
				break;
			}
			i++;
		}
		return year;
	}
	
	private static Map<String, String> read(File map) throws Exception {
		Map<String, String> mapping = new HashMap<String, String>();
		if (map.exists()) { 
			BufferedReader reader = new BufferedReader(new FileReader(map));
			String line;
			while ((line = reader.readLine()) != null) {
				String[] array = line.trim().split("\t");
				if (array.length == 3)
					mapping.put(array[0] + " # " + array[2], array[1]);
			}
			reader.close();
		}
		return mapping;
	}
	
	private static void write(File map, Map<String, List<List<String>>> references, Map<String, String> mapping) throws Exception {
		BufferedWriter writer = new BufferedWriter(new FileWriter(map));
		List<String> keys = new LinkedList<String>(references.keySet());
		Collections.sort(keys, new Comparator<String>() {
			@Override
			public int compare(String fst, String snd) {
				return fst.replaceAll("[a-z]", "").compareTo(snd.replaceAll("[a-z]", ""));
			}
		});
		for (String key: keys)
			if (references.get(key).size() > 1) {
				List<String> lines = new LinkedList<String>();
				for (List<String> reference: references.get(key)) {
					String title = title(reference);
					lines.add(key + "\t" + mapping.get(key + " # " + title) + "\t" + title);
				}
				Collections.sort(lines);
				for (String line: lines) {
					writer.write(line);
					writer.newLine();
				}
			}
		writer.flush();
		writer.close(); 
	}
	
}
